/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: Aver TVPhone98 w/VCR remote control driver. WORKING !!!!!!
              Version 2.0 - Updated key definitions
              Version 3.0 - Updated key pauses and codes control
              Version 3.1 - Added released keys control (as example)
              Version 3.2 - Simpified keys detection
              Version 3.3 - Reading key delays from "jtv.ini" file
			  Version 3.4 - Updated key definitions

 Prototype (c) 1999 by Mirosaw Grybo   <mgrybos@postinfo.com.pl>
 Copyright (c) 2000 by Ivan Kolin  <ivkolin@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

// Key codes from: Ivan Kolin, <ivkolin@mailru.com>

// Firm    : AVerMedia TECHNOLOGIES Inc.
// Kit Type: Retail  
// Mark    : AVerMedia
// Model   : TVPhone98 w/VCR
// Tuner   : Philips
// Standart: SECAM
// System  : DK

// btTool.exe display for "AverMedia TVPhone98 w/VCR":
// GPIO_Dma_Ctl - 00000004;
// GPIO_Out_En  - 00007000;
// GPIO_Data    - remarked column (for pressed -- released
//                keys and closed JTV-window).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 00xxxxc (when JTV-window is open)


KeyDef keys[]=
{
// 1-st row keys layout                   Pressed     Released
        { 0x85, "Key_TV/FM", "", },     //00857fxx -- 00847fxx
        { 0x45, "Key_CD", "", },        //00457fxx -- 00447fxx
        { 0xc5, "Key_Teletext", "", },  //00c57fxx -- ...
        { 0x05, "Key_Power", "Power", },//00057fxx
// 2-nd row keys layout
        { 0xa5, "Key_1", "1", },        //00a57fxx
        { 0x65, "Key_2", "2", },        //...
        { 0xe5, "Key_3", "3", },
        { 0x25, "Key_Video", "", },
// ...
        { 0x95, "Key_4", "4", },
        { 0x55, "Key_5", "5", },
        { 0xd5, "Key_6", "6", },
        { 0x15, "Key_Audio", "", },

        { 0xb5, "Key_7", "7", },
        { 0x75, "Key_8", "8", },
        { 0xf5, "Key_9", "9", },
        { 0x35, "Key_FullScreen", "", },

        { 0x8d, "Key_0", "0", },
        { 0x4d, "Key_Display", "", },
        { 0xcd, "Key_Loop", "Recall", },
        { 0x0d, "Key_Preview", "", },

        { 0xad, "Key_AutoScan", "", },
        { 0x6d, "Key_Freeze", "", },
        { 0xed, "Key_Capture", "", },
        { 0x2d, "Key_Mute", "", },

        { 0x9d, "Key_Record", "", },
        { 0x5d, "Key_Pause", "", },
        { 0xdd, "Key_Stop", "", },
        { 0x1d, "Key_Play", "", },

        { 0xbd, "Key_Red", "", },
        { 0x7d, "Key_VolumeDown", "VolumeDown", },//...
        { 0xfd, "Key_VolumeUp", "VolumeUp", },    //00fd7fxx -- 00fc7fxx
        { 0x3d, "Key_Yellow", "", },              //003d7fxx -- 003c7fxx
 
        { 0x87, "Key_Green", "", },                 //0085ffxx -- 0084ffxx
        { 0x47, "Key_ChannelDown", "ChannelDown", },//0045ffxx -- 0044ffxx
        { 0xc7, "Key_ChannelUp", "ChannelUp", },    //00c5ffxx -- 00c4ffxx
        { 0x07, "Key_Cyan", "", },                  //0005ffxx -- 0004ffxx
		  	   
// When no one key is pressed, GPIO_Data keeps unpressed key code
// for previously pressed key, that is less by 1 (0x00010000).

// Here is example definitions for unpressed (released) arrow-keys
// of the remote.

//        { 0xa4, "RKey_1", "", },      //00a47fxx
//        { 0x64, "RKey_2", "", },      //...
//        { 0xe4, "RKey_3", "", },
      
//        { 0x94, "RKey_4", "", },
//        { 0xd4, "RKey_6", "", },
     
//        { 0xb4, "RKey_7", "", },
//        { 0x74, "RKey_8", "", },
//        { 0xf4, "RKey_9", "", },

};

typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=1000;	// default first delay for a key
INT repeat_delay=100;	// default repeat delay for a key

void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
		{
			log("> Initializing AVerMedia TVPhone98 w/VCR remote module v3.4.\n");
            log("> Copyright (c) 2000 by Ivan Kolin, <ivkolin@mailru.com>.\n");
			log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
		}; 


        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();
   
                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    INT key_pause=20; // new key initial delay
				    DWORD press_flag=0;
					DWORD extkey_flag=0;
				    //DWORD test_code=0; // for test block


                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0x0004);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0x00007000));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | 0x7ff3);
                   
				  	   
                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;

                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
 						                         
						 //press_flag = ((gpio_data & 0x00010000)>>16); // for test block
						 extkey_flag = ((gpio_data & 0x00008000)>>12);
						 value = ((gpio_data &0x00ff0000)>>16) + (extkey_flag/4);

		 				// --- Test block begin - comment all while build checked out driver
						//if (test_code != value)
						//{
						//	log(">> ---- Test information ---- \n");
						//	log(">>      Tick_Counter: %d\n",GetTickCount());
						//	log(">>         GPIO_DATA: %x\n",gpio_data);
						//	log(">>  Current_Key_Code: %x\n",value);
						//	log(">> Previous_Key_Code: %x\n",last_code);
						//	log(">>    Press_Key_Flag: %x\n",press_flag);
						//	log(">>      Ext_Key_Flag: %x\n",extkey_flag);
						//	test_code = value;
						//}
           				// --- Test block end
           
                         if (gpio_data & 0x00010000) // any key is pressed
                         {	 
                             if (last_code == value)
                             {  // auto repeat for any pressed key
                                //log(">>    Press_Key_Auto: %x\n",value); //for test block

                            	AddStroke(last_code);
								key_pause = repeat_delay;
                             } 
                             else
                             {	// first run for any pressed key
								//log(">>   Press_Key_First: %x\n",value); //for test block
 
                                last_code = value;
								AddStroke(last_code);
                                key_pause = first_delay;
                             }
                         }
                         else  // no one key is pressed
                         {	  
                             if ((value!=last_code) && (last_code!=0xffffffff))
                             {
                                last_code = value;
								
                             }
							   
                             // Example_Begin 
                             // Released key activation when JTV-window absent
                             // and any key has been pressed before
                             // else if (((gpio_data &0x0000000f)!=0xc) && (last_code!=0xffffffff))
                             // {
                             // 	 AddStroke(value); // Activation for defined RKey_XX
                             // }
                             // Example_End    
								   
                                key_pause = 20;
                         };
				
                    };

                log("> Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};


